
from flask import request

from seed.schema.base import BaseSchema
from seed.api.endpoints._base import RestfulBaseView, HttpMethods

from seed.api.endpoints.panels import PanelSchema, PanelsModel
from seed.api.endpoints.filters import FilterSchema, FiltersModel
from seed.api.endpoints.databases import DatabasesSchema

from seed.models._base import session
from seed.models.databases import Databases

from seed.utils.auth import api_require_login
from seed.utils.database import get_db_instance

from seed.libs.data_access.app import DataAccess
from seed.libs.filter_access.app import FilterAccess


class QueryData(RestfulBaseView):
    """ 获取数据
    """
    url = 'query_data'
    decorators = [api_require_login]

    access_methods = [HttpMethods.POST]

    def post(self, panel_id=None):
        try:
            query_params = request.get_json()
        except:
            query_params = {}

        if panel_id:
            panel_data = self.session.query(PanelsModel).filter_by(id=panel_id).first()
            panel_data, errors = PanelSchema(exclude=PanelsModel.column_filter).dump(panel_data)
            if errors:
                return self.response_json(self.HttpErrorCode.ERROR, msg=str(errors))
        else:
            panel_data = {}

        panel_data.update(query_params)

        try:
            dtype, db = get_db_by_id(panel_data['db_source'])
            query_datas = DataAccess(dtype, db, **panel_data).get_datas()
        except Exception as e:
            error_message = str(e)
            return self.response_json(self.HttpErrorCode.ERROR, msg=error_message)

        return self.response_json(self.HttpErrorCode.SUCCESS, data=query_datas)


class QueryFilters(RestfulBaseView):
    """ 获取Filters数据
    """
    url = 'query_filters'
    decorators = [api_require_login]

    access_methods = [HttpMethods.POST]

    def post(self, filter_id=None):
        try:
            query_params = request.get_json()
        except:
            query_params = {}

        if filter_id:
            filter_data = self.session.query(FiltersModel).filter_by(id=filter_id).first()
            filter_data, errors = FilterSchema(exclude=FiltersModel.column_filter).dump(filter_data)
            if errors:
                return self.response_json(self.HttpErrorCode.ERROR, msg=str(errors))
        else:
            filter_data = {}

        filter_data.update(query_params)

        if not filter_data:
            return self.response_json(self.HttpErrorCode.ERROR, msg='过滤设置不能为空')

        if filter_data['condition_type'] in ('sql'):
            try:
                dtype, db = get_db_by_id(filter_data['db_source'])
            except Exception as e:
                error_message = str(e)
                return self.response_json(self.HttpErrorCode.ERROR, msg=error_message)

            filter_data['conditions'] = FilterAccess(db, filter_data['conditions'], filter_data.get('query', {})).query_datas()

        return self.response_json(self.HttpErrorCode.SUCCESS, data=filter_data)


def get_db_by_id(db_source):
    db_data = session.query(Databases).filter_by(id=db_source).first()
    db_conf, errors = DatabasesSchema().dump(db_data)
    if errors:
        raise Exception(errors)

    db = get_db_instance(**db_conf)
    return db_conf['dtype'], db


from seed.models.bussiness import Bussiness
from seed.models.bmanager import BManager
from seed.models.buser import BUser
from seed.models._base import session


def get_permission_datas_by_user(user):
    all_datas = session.query(Bussiness).all()
    all_datas = [data.row2dict() for data in all_datas]

    if user.role == 'super_admin':
        # 所有权限
        for data in all_datas:
            data.update({'edit': True, 'delete': True})
        permission_datas = all_datas
        un_permission_datas = []

    else:
        # 管理的业务
        order_datas = session.query(Bussiness)\
            .join(BManager, Bussiness.id == BManager.bussiness_id)\
            .filter(BManager.user_id == user.id)\
            .all()
        order_ids = [data.id for data in order_datas]
        order_datas = [data.row2dict() for data in order_datas]
        for data in order_datas:
            data.update({'edit': True, 'delete': False})

        # 有权限的业务
        permission_datas = session.query(Bussiness)\
            .join(BUser, Bussiness.id == BUser.bussiness_id)\
            .filter(BUser.user_id == user.id)\
            .all()
        permission_ids = [data.id for data in permission_datas]
        permission_datas = [data.row2dict() for data in permission_datas]
        for data in permission_datas:
            data.update({'edit': False, 'delete': False})

        permission_datas = [
            data for data in permission_datas if data['id'] not in order_ids
        ]
        permission_datas = order_datas + permission_datas
        permission_ids = [data['id'] for data in permission_datas]

        un_permission_datas = [
            data for data in all_datas if data['id'] not in permission_ids
        ]
    return permission_datas, un_permission_datas


def has_bussiness_permission(user, bussiness_id):
    permission_datas, _ = get_permission_datas_by_user(user)
    return any([bussiness_id == bussiness['id'] for bussiness in permission_datas])



class FilterAccess(object):
    def __init__(self, db, conditions, query,  *args, **kwargs):
        self.db_instance = db
        self.sql = conditions
        self.query = query

    def query_datas(self):
        for key, value in self.query.items():
            if isinstance(value, list):
                self.query[key] = '(%s)' % ', '.join([str(item) for item in value])

        sql = self.sql.format(**self.query)
        print(sql)

        rows = self.db_instance.query(sql)
        return rows



from seed.drives.impala import Impala
from seed.drives.mysql import MySQL
from seed.drives.postgresql import PostgreSQL


ALL_DRIVES = {
    # 'impala': Impala,
    'mysql': MySQL,
    'postgresql': PostgreSQL
}


import os
from pathlib import Path
import time

local_file_path = os.path.join(
    Path(os.path.dirname(os.path.realpath(__file__))).parent, 'static', 'files'
)


class LocalFile(object):
    def save(self, file):
        filename = '_'.join([str(int(time.time())), file.filename])
        upload_path = os.path.join(local_file_path, filename)

        if not os.path.exists(local_file_path):
            os.makedirs(local_file_path)

        try:
            file.save(upload_path)
        except Exception as e:
            print(e)
            return None

        return os.path.join('files', filename)



import os
from pathlib import Path

from flask import Blueprint, render_template, send_from_directory


bp = Blueprint('front', __name__)

static_folder_path = os.path.join(
    Path(os.path.dirname(os.path.realpath(__file__))).parent, 'static'
)

@bp.route('/', defaults={'path': ''})
@bp.route('/<path:path>')
def index(path):
    """
    如果是静态文件则利用seed_from_directory来进行加载
    """
    if path and os.path.exists(os.path.join(static_folder_path, path)):
        return send_from_directory(static_folder_path, path)

    return render_template('index.html')


from inspect import isclass

from flask import Blueprint

from seed import api
from seed.utils.helper import get_package_members, get_immediate_cls_attr
from seed.api.common import _MethodView


def register_api(app):
    """ register api interface
    """

    predicate = lambda m: isclass(m) and issubclass(m, _MethodView) and not get_immediate_cls_attr(m, '__abstract__')
    members = get_package_members(api, predicate, '')
    for pre_url, handlers in members.items():
        bp = Blueprint(pre_url, __name__)
        for handler in set(handlers):
            bp = handler.register_api(bp)

        app.register_blueprint(bp, url_prefix=pre_url)



from flask import request, g
from seed.cache.user_bussiness import UserBussinessCache
from seed.api.endpoints._base import RestfulBaseView

from seed.utils.permissions import has_bussiness_permission

from seed.utils.auth import api_require_login


class BUserSelect(RestfulBaseView):
    """ 用户当前选择业务
    """
    decorators = [api_require_login]

    def get(self):
        current_bussiness_id = g.bussiness_id
        return self.response_json(
            self.HttpErrorCode.SUCCESS,
            data={'bussiness_id': current_bussiness_id}
        )

    def post(self):
        input_json = request.get_json()
        bussiness_id = input_json['bussiness_id']
        if not has_bussiness_permission(g.user, bussiness_id):
            return self.response_json(self.HttpErrorCode.FORBIDDEN)

        UserBussinessCache().set(g.user.id, bussiness_id)
        return self.response_json(self.HttpErrorCode.SUCCESS)


from sqlalchemy import and_
from flask import current_app, g

from seed.schema.base import BaseSchema
from seed.api.endpoints._base import RestfulBaseView, HttpMethods
from seed.models.account import Account as AccountModel
from seed.models.role import Role
from seed.models.buserrole import BUserRole
from seed.models.bmanager import BManager
from seed.models.rolemenu import RoleMenu
from seed.models.menu import Menu as MenuModel
from seed.utils.auth import api_require_login, require_admin


class UserSchema(BaseSchema):
    class Meta:
        model = AccountModel


class User(RestfulBaseView):
    """ 用户相关
    """
    model_class = AccountModel
    schema_class = UserSchema
    decorators = [api_require_login]

    access_methods = [HttpMethods.GET]

    def get(self):
        """ 获取用户信息, 如果是SSO的校验
        成功后自动添加用户信息到用户列表
        """
        user = g.user.row2dict()
        user['brole'] = self._get_role(g.user.id)

        return self.response_json(self.HttpErrorCode.SUCCESS, data=user)

    def _get_role(self, uid):
        roles = self.session.query(Role.role)\
            .join(BUserRole, and_(
                BUserRole.role_id == Role.id,
                BUserRole.user_id == uid,
                BUserRole.bussiness_id == g.bussiness_id)
            ).all()
        return roles


class UserMenu(RestfulBaseView):
    """ 获取当前用户的菜单
    """
    url = '/user/menu'

    decorators = [api_require_login]

    access_methods = [HttpMethods.GET]

    def get(self):
        if require_admin():
            # 如果是业务管理员以上，返回当前业务的所有菜单
            query_session = self.session.query(MenuModel, "True").filter(MenuModel.bussiness_id==g.bussiness_id)
        else:
            # 其他角色通过关联用户角色表来获取到当前的菜单
            query_session = self.session.query(MenuModel, RoleMenu.role_permission)\
                .join(RoleMenu, and_(MenuModel.id==RoleMenu.menu_id, RoleMenu.bussiness_id==g.bussiness_id))\
                .join(BUserRole, and_(RoleMenu.role_id==BUserRole.role_id, BUserRole.bussiness_id==g.bussiness_id, BUserRole.user_id==g.user.id))

        menu_data = query_session.all()
        menus = self._encode_menus(menu_data)

        return self.response_json(self.HttpErrorCode.SUCCESS, data=menus)

    def _encode_menus(self, menus):
        menu_data = {}

        for menu, role_permission in menus:
            menu_data['-'.join([str(menu.parent_id), str(menu.left_id)])] = menu.row2dict()
            menu_data['-'.join([str(menu.parent_id), str(menu.left_id)])]['role_permission'] = role_permission

        menus = {'id': 0}
        middle_menu = [menus]
        while middle_menu:
            current_menu = middle_menu.pop()
            parent_id, left_id = current_menu['id'], 0

            while True:
                current_key = '-'.join([str(parent_id), str(left_id)])
                if current_key not in menu_data:
                    break
                if menu_data[current_key]['role_permission']:
                    current_menu.setdefault('sub_menus', []).append(menu_data[current_key])

                middle_menu.append(menu_data[current_key])

                left_id = menu_data[current_key]['id']

        return menus.get('sub_menus', [])



from ._base import *
from .account import *
from .bussiness import *
from .databases import *
from .menu import *
from .role import *
from .bmanager import *
from .buser import *
from .panels import *
from .filters import *

from .analogdatas import *



from seed.libs.data_access.dataware.base import DataModel
from seed.libs.data_access.middledata.base import MiddleData
from seed.libs.data_access.formatters.base import FormatterFactory


class DataAccess(object):
    def __init__(self, dtype, db, *args, **kwargs):
        # self.dtype = dtype
        self.db = db

        self.sql = kwargs['sql']
        if not self.sql:
            raise Exception("SQL不能为空！")

        self.indexs = kwargs['indexs']
        if not self.indexs:
            raise Exception("指标不能为空！")

        self.dimensions = kwargs['dimensions']
        if not self.dimensions:
            raise Exception("维度不能为空！")

        self.query = kwargs.get('query', {})

        self.charttype = kwargs.get('charttype', 'chart')

        self.format_args = {'vth_columns': kwargs.get('vth_columns', [])}

    def get_datas(self):
        # 获取原数据
        try:
            source_data, sql_info = self.get_source_data()
            middle_data = self.transfer_middle_data(source_data)
        except Exception as e:
            raise Exception('数据库取数错误: %s' % str(e))

        # 格式化
        try:
            format_data = self.format_data(middle_data)
        except Exception as e:
            raise Exception('数据格式化出错: %s' % str(e))

        return format_data

    def get_source_data(self):
        source_data, sql_info = DataModel(
            self.db, self.sql, self.query, self.dimensions, self.indexs
        ).query_data()
        return source_data, sql_info

    def transfer_middle_data(self, source_data):
        middle_data = MiddleData(
            source_data, self.dimensions, self.indexs
        ).convert()
        return middle_data

    def format_data(self, middle_data):

        formatter = FormatterFactory(self.charttype).formatter_class()
        format_data = formatter(
            self.indexs, self.dimensions, middle_data, self.format_args
        ).format_data()

        return format_data



import json
from datetime import datetime

from flask import g
from flask_migrate import Migrate
from flask_marshmallow import Marshmallow
from sqlalchemy.sql import sqltypes
from sqlalchemy.ext.declarative import declared_attr

from flask_sqlalchemy import SQLAlchemy, BaseQuery, orm

from seed.utils.time import convert_utc_to_local

__all__ = ['db', 'ma', 'migrate', 'session', 'BaseModel', 'BussinessModel']


DEFAULT_DATETIME = '1970-01-01 00:00:00'


class SeedQuery(BaseQuery):
    def __init__(self, *args, **kwargs):
        super(SeedQuery, self).__init__(*args, **kwargs)

    def as_list(self, *clolumns):
        return [{key: getattr(row, key, None) for key in row.keys()} for row in self]


class SessionMixin(object):
    column_filter = []

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def save(self):
        db.session.add(self)
        db.session.commit()
        return self

    def flush(self):
        db.session.flush(self)
        db.session.commit()
        return self

    def delete(self):
        db.session.delete(self)
        db.session.commit()
        return self

    def row2dict(self):
        d = {}
        for column in self.__table__.columns:
            if column.name in self.column_filter:
                continue

            if isinstance(column.type, sqltypes.DateTime):
                local_time = convert_utc_to_local(getattr(self, column.name), 'Asia/Shanghai')
                d[column.name] = local_time.strftime('%Y-%m-%d %H:%M:%S') if local_time else DEFAULT_DATETIME
            else:
                d[column.name] = getattr(self, column.name)
        return d


db = SQLAlchemy(query_class=SeedQuery)
migrate = Migrate()
session = orm.scoped_session(orm.sessionmaker(autocommit=True))
ma = Marshmallow()


class BaseModel(db.Model, SessionMixin):
    __abstract__ = True
    column_filter = ['created', 'updated']

    id = db.Column(db.Integer, primary_key=True)
    created = db.Column(
        db.DateTime, nullable=False, default=datetime.utcnow()
    )
    updated = db.Column(
        db.DateTime, default=datetime.utcnow(),
        onupdate=datetime.utcnow()
    )

    def __init__(self, *args, **kwargs):
        super(BaseModel, self).__init__(*args, **kwargs)

    def delete(self):
        if hasattr(self, 'status'):
            self.status = -1
            self.save()
        else:
            super(BaseModel, self).delete()


class BussinessModel(BaseModel):
    """ 与业务绑定的Model
    """
    __abstract__ = True
    column_filter = ['created', 'updated', 'bussiness_id']

    @declared_attr
    def bussiness_id(cls):
        return db.Column(db.Integer, db.ForeignKey('bussiness.id'))

    def __init__(self, *args, **kwargs):
        self.bussiness_id = g.bussiness_id
        super(BussinessModel, self).__init__(*args, **kwargs)


class PageModel(BussinessModel):
    """ 与页面绑定的Model
    """
    __abstract__ = True

    column_filter = ['created', 'updated', 'bussiness_id', 'page_id']

    @declared_attr
    def page_id(cls):
        return db.Column(db.Integer, db.ForeignKey('menu.id'))


from flask import current_app

from seed.drives.email import Email


def send_active_email(to_mail, active_url, redirect_url):
    """ 发送激活邮件
    """
    title = '新用户激活'
    mail_content = """
        亲爱的用户:
            感谢你使用Seed自定义报表系统，请点击以下链接激活用户:
            {active_url}?redirect_url={redirect_url}
            此链接24小时之后失效，请尽快激活邮箱。
        Seed团队
    """.format(active_url=active_url, redirect_url=redirect_url)
    Email(current_app.config).send_mail([to_mail], title, mail_content)


def send_reset_password_email(to_mail, active_url, redirect_url):
    """ 发送重置密码邮件
    """
    title = '密码重置'
    mail_content = """
        亲爱的用户:
            你的密码重置连接如下:
            {active_url}?redirect_url={redirect_url}
            此链接24小时之后失效，请尽快点击重置密码。
        Seed团队
    """.format(active_url=active_url, redirect_url=redirect_url)
    Email(current_app.config).send_mail([to_mail], title, mail_content)



import decimal

import pymysql
from datetime import datetime, date

from seed.drives.postgresql import PostgreSQL


class MySQL(PostgreSQL):
    def query(self, sql, params=[], retry_count=1):
        """ 查询SQL语句
        """
        self._raise_retry_count(retry_count)

        cursor = self._gen_cursor()
        try:
            query_data = self._query(cursor, sql, params)
            return query_data
        except (pymysql.OperationalError, pymysql.DatabaseError, pymysql.InterfaceError) as e:
            if retry_count < 1:
                raise
            self._connect()
            return self.query(sql, params=params, retry_count=retry_count-1)
        finally:
            cursor.close()

        return []

    def _query(self, cursor, sql, params):
        self._execute(cursor, sql, params=params)
        datas = list(cursor.fetchall())
        return self._replace_type(datas)

    def _execute(self, cursor, sql, params=[]):
        """ 执行SQL
        """
        try:
            cursor.execute(sql, params)
            self._commit()
        except pymysql.ProgrammingError as e:
            self._rollback()
            raise Exception(str(e))
        except (pymysql.OperationalError, pymysql.DatabaseError, pymysql.InterfaceError) as e:
            raise
        except Exception as e:
            self._rollback()
            raise

    def _connect(self):
        try:
            self.conn = pymysql.connect(
                db=self.name,
                user=self.user,
                password=self.password,
                host=self.ip,
                port=self.port,
                charset='utf8',
                cursorclass=pymysql.cursors.DictCursor
            )
        except Exception as e:
            raise

    def _replace_type(self, datas):
        """ 替换数据类型
        """
        for data in datas:
            for key, value in data.items():
                if isinstance(value, datetime):
                    data[key] = value.strftime('%Y-%m-%d')
                if isinstance(value, date):
                    data[key] = value.strftime('%Y-%m-%d')
                if isinstance(value, decimal.Decimal):
                    data[key] = int(value)
        return datas



from inspect import isclass

from seed.libs.data_access.utils.auto_register import get_package_members, get_immediate_cls_attr


class FormatterFactory(object):
    registerd_charttypes = {}

    def __init__(self, charttype):
        self.charttype = charttype

    def formatter_class(self):
        from seed.libs.data_access.formatters.line import LineFormatter
        if self.charttype not in self.registerd_charttypes:
            self._register_charttypes()

        return self.registerd_charttypes.get(self.charttype.lower()+'formatter', LineFormatter)

    def _register_charttypes(self):
        from seed.libs.data_access import formatters
        predicate = lambda m: isclass(m) and issubclass(m, BaseFormatter) and not get_immediate_cls_attr(m, '__abstract__')
        members = get_package_members(formatters, predicate)
        self.registerd_charttypes = {member.__name__.lower(): member for member in members}


class BaseFormatter(object):
    __abstract__ = True

    def __init__(self, indexs, dimensions, data, format_args):
        self.indexs = indexs
        self.dimensions = dimensions
        self.data = data
        self.format_args = format_args



from ._base import db, BussinessModel

# from seed.models import Panels, Filters

__all__ = ["Menu",]


class Menu(BussinessModel):
    name = db.Column(db.Text, nullable=False)

    parent_id = db.Column(db.Integer, default=0)
    left_id = db.Column(db.Integer, default=0)

    panels = db.relationship('Panels', cascade="all,delete", backref="menu")
    filters = db.relationship('Filters', cascade="all,delete", backref="menu")


from seed.libs.data_access.formatters.line import LineFormatter


class BarFormatter(LineFormatter):
    def __init__(self, *args, **kwargs):
        super(LineFormatter, self).__init__(*args, **kwargs)


import click


class AddressParamType(click.ParamType):
    name = 'address'

    def __call__(self, value, param=None, ctx=None):
        if value is None:
            return (None, None)
        return self.convert(value, param, ctx)

    def convert(self, value, param, ctx):
        if ':' in value:
            host, port = value.split(':', 1)
            port = int(port)
        else:
            host = value
            port = None
        return host, port


Address = AddressParamType()


@click.group()
def run():
    "Run a service."

@run.command()
@click.option(
    '--bind', '-b', default=None, help='Bind address.', type=Address
)
@click.option(
    '--workers', '-w', default=0,
    help='The number of worker process for handling requests'
)
@click.option(
    '--debug', '-d', default=False,
    help='The debug option for web'
)
def web(bind, workers, debug):
    from seed.services.app import SeedHttpServer
    from seed.runner.setting import discover_configs

    _, config_file, _ = discover_configs()
    http_server = SeedHttpServer(
        host=bind[0],
        port=bind[1],
        workers=workers,
        config_file=config_file
    )
    http_server.run()



import inspect

from flask import request, g
from marshmallow import ValidationError

from seed.models import db
from seed.api.common import _MethodView
from seed.models._base import BussinessModel

RESTFUL_METHODS = ['GET', 'POST', 'PUT', 'DELETE']


class HttpMethods(object):
    GET = 'GET'
    POST = 'POST'
    PUT = 'PUT'
    DELETE = 'DELETE'


class RestfulBaseView(_MethodView):
    """ BaseView for Restful style
    """
    __abstract__ = True

    pk_type = 'string'

    session = db.session

    model_class = None
    schema_class = None

    access_methods = [HttpMethods.GET, HttpMethods.POST, HttpMethods.PUT, HttpMethods.DELETE]

    def __init__(self, *args, **kwargs):
        super(RestfulBaseView, self).__init__(*args, **kwargs)

    def get(self, model_id=None):
        """ GET
        GET /base
        get paragraph node list

        GET /base/<model_id>
        get single node which id is model_id

        Arguments:
            model_id {int} -- resource id
        """
        query_session = self.session.query(self.model_class)

        if issubclass(self.model_class, BussinessModel):
            query_session = query_session.filter_by(bussiness_id=g.bussiness_id)

        if model_id:
            data = query_session.filter_by(id=model_id).first()
            data = self.schema_class(exclude=self.model_class.column_filter).dump(data)
        else:
            data = query_session.all()
            data = self.schema_class(many=True, exclude=self.model_class.column_filter).dump(data)

        return self.response_json(self.HttpErrorCode.SUCCESS, data=data.data)

    def post(self):
        """ POST
        """
        input_json = request.get_json()

        if isinstance(input_json, list):
            schema = self.schema_class(many=True)
        else:
            schema = self.schema_class()
        datas, errors = schema.load(input_json)

        if errors:
            return self.response_json(self.HttpErrorCode.PARAMS_VALID_ERROR, msg=errors)

        if isinstance(datas, list):
            [data.save() for data in datas]
        else:
            datas.save()
        return self.response_json(self.HttpErrorCode.SUCCESS)

    def put(self, model_id=None):
        """ PUT

        Arguments:
            model_id {int} -- resource id
        """
        input_json = request.get_json()
        if model_id:
            datas, errors = self.schema_class().load(
                input_json, instance=self.model_class.query.get(model_id), partial=True
            )
            if errors:
                return self.response_json(self.HttpErrorCode.PARAMS_VALID_ERROR, msg=errors)
            datas.save()
            datas = self.schema_class().dump(datas)
        else:
            datas, errors = self.schema_class().load(input_json, many=True)
            if errors:
                return self.response_json(self.HttpErrorCode.PARAMS_VALID_ERROR, msg=errors)

            [data.save() for data in datas]
            datas = self.schema_class().dump(datas, many=True)

        return self.response_json(self.HttpErrorCode.SUCCESS, data=datas.data)

    def delete(self, model_id):
        """ DELETE

        Arguments:
            model_id {int} -- resource id
        """
        data = self.model_class.query.get(model_id)
        if data:
            data.delete()
            return self.response_json(self.HttpErrorCode.SUCCESS, 'Success!')
        return self.response_json(self.HttpErrorCode.ERROR, 'The data is not exists!')

    @classmethod
    def register_api(cls, app):
        if hasattr(cls, 'url'):
            url = cls.url or '/' + cls.__name__.lower()
        else:
            url = cls.__name__.lower()

        view_func = cls.as_view(cls.__name__.lower())

        for method in RESTFUL_METHODS:
            if method not in cls.access_methods:
                continue

            method_params = inspect.signature(getattr(cls, method.lower())).parameters
            defaults = {
                param_key: param_value.default
                for param_key, param_value in method_params.items()
                if param_key != 'self' and param_value.empty is not param_value.default
            }
            pk = list(method_params.keys())[1] if len(method_params.keys()) > 1 else ''

            if defaults:
                app.add_url_rule(
                    url, defaults=defaults,
                    view_func=view_func,
                    methods=[method],
                )
                app.add_url_rule(
                    '%s/<%s:%s>' % (url, cls.pk_type, pk),
                    view_func=view_func,
                    methods=[method]
                )
            elif len(method_params) > 1:
                app.add_url_rule(
                    '%s/<%s:%s>' % (url, cls.pk_type, pk),
                    view_func=view_func,
                    methods=[method]
                )
            else:
                app.add_url_rule(
                    url,
                    view_func=view_func, methods=[method]
                )

        return app



from flask import request, g
from sqlalchemy import and_

from seed.schema.base import BaseSchema
from seed.api.endpoints._base import RestfulBaseView, HttpMethods
from seed.models import BUser as BUserModel
from seed.models import Account as AccountModel
from seed.models import BManager as BManagerModel

from seed.models.role import Role
from seed.models.buserrole import BUserRole

from seed.utils.auth import api_require_admin


class BUserSchema(BaseSchema):
    class Meta:
        model = BUserModel
        include_fk = True


class Buser(RestfulBaseView):
    """ 用户业务映射关系添加
    """
    model_class = BUserModel
    schema_class = BUserSchema
    decorators = [api_require_admin]

    def get(self, bussiness_id):
        users = self.session.query(BUserModel.id, AccountModel)\
            .join(AccountModel, AccountModel.id == BUserModel.user_id)\
            .filter(BUserModel.bussiness_id == bussiness_id)\
            .all()

        datas = []
        for user in users:
            data = user.Account.row2dict()
            data['account_id'], data['id'] = data['id'], user.id
            data['brole'] = self._get_role(data['id'])

            datas.append(data)

        return self.response_json(self.HttpErrorCode.SUCCESS, data=datas)

    def _get_role(self, uid):
        roles = self.session.query(Role)\
            .join(BUserRole, and_(
                BUserRole.role_id == Role.id,
                BUserRole.user_id == uid,
                BUserRole.bussiness_id == g.bussiness_id)
            ).all()
        roles = [role.row2dict() for role in roles]
        return roles


class UnBuserList(RestfulBaseView):
    """ 不属于当前业务的用户名单
    """
    url = 'un_busers'
    decorators = [api_require_admin]
    access_methods = [HttpMethods.GET]

    def get(self):

        in_user_query = self.session.query(BUserModel.user_id)\
            .filter(BUserModel.bussiness_id == g.bussiness_id)

        b_managers_query = self.session.query(BManagerModel.user_id)\
            .filter(BManagerModel.bussiness_id == g.bussiness_id)

        users = self.session.query(AccountModel)\
            .filter(~AccountModel.id.in_(in_user_query))\
            .filter(~AccountModel.id.in_(b_managers_query))

        users = [user.row2dict() for user in users]
        return self.response_json(self.HttpErrorCode.SUCCESS, data=users)



import json

from flask import request, g
from marshmallow import pre_load, post_dump

from seed.schema.base import BaseSchema
from seed.api.endpoints._base import RestfulBaseView

from seed.models.panels import Panels as PanelsModel
from seed.utils.auth  import api_require_admin


class PanelSchema(BaseSchema):
    class Meta:
        model = PanelsModel
        include_fk = True

    @post_dump(pass_many=True)
    def dumps_indexs(self, in_data, many):
        if many:
            for data in in_data:
                data['indexs'] = json.loads(data['indexs'])
                data['dimensions'] = json.loads(data['dimensions'])
        else:
            in_data['indexs'] = json.loads(in_data['indexs'])
            in_data['dimensions'] = json.loads(in_data['dimensions'])

        return in_data

    @pre_load
    def loads_indexs(self, out_data):
        out_data['indexs'] = json.dumps(out_data['indexs'])
        out_data['dimensions'] = json.dumps(out_data['dimensions'])
        return out_data


class Panels(RestfulBaseView):
    """ 数据面板
    """
    model_class = PanelsModel
    schema_class = PanelSchema
    decorators = [api_require_admin]


from flask import request

from seed.api.endpoints._base import RestfulBaseView, HttpMethods
from seed.drives import ALL_DRIVES


class DatabaseTest(RestfulBaseView):
    url = 'database_test'

    access_methods = [HttpMethods.POST]

    def post(self):
        db_conf = request.get_json()

        try:
            drive = ALL_DRIVES[db_conf['dtype']]
            drive_instance = drive(
                db_conf['ip'], int(db_conf['port']), db_conf['name'],
                db_conf['user'], db_conf['password']
            )
            success = drive_instance.test_connection()
            message = '数据库连接成功!'
        except Exception as e:
            success = False
            message = str(e)

        if success:
            return self.response_json(self.HttpErrorCode.SUCCESS, msg=message)

        return self.response_json(self.HttpErrorCode.ERROR, msg=message)


from flask  import jsonify

class HttpErrorCode(object):
    SUCCESS = 200
    ERROR = 400
    UNAUTHORIZED = 401
    FORBIDDEN = 403
    NOT_FUND = 404
    AUTHORIZED_ERROR = 420
    PARAMS_VALID_ERROR = 421


ERROR_DICT = {
    HttpErrorCode.SUCCESS: 'SUCCESS!',
    HttpErrorCode.UNAUTHORIZED: 'Unauthorized!',
    HttpErrorCode.PARAMS_VALID_ERROR: 'Params validation error!',
    HttpErrorCode.FORBIDDEN: 'No permission!',
    HttpErrorCode.NOT_FUND: 'Not found!',
    HttpErrorCode.AUTHORIZED_ERROR: 'Password is valid!',
}


def response(code, message=None, data={}):
    response_content = {'code': code, 'data': data}
    response_content['message'] = message if message else ERROR_DICT.get(code, 'Unknown error code!')
    return response_content


def response_json(code, msg=None, data={}):
    return jsonify(response(code, msg, data))



from inspect import getmembers


def get_module_members(modules, predicate):
    members = getmembers(modules, predicate)
    return map(lambda m: m[1], members)


def get_package_members(package, predicate):
    from pkgutil import iter_modules
    from importlib import import_module

    members = []
    for _, name, ispkg in iter_modules(package.__path__, package.__name__+'.'):
        if ispkg:
            sub_members = get_package_members(
                import_module(name), predicate
            )
            members.append(sub_members)

        members.extend(get_module_members(import_module(name), predicate))

    return members


def get_immediate_cls_attr(cls, attrname):
    if not issubclass(cls, object):
        return None

    for base in cls.__mro__:
        if attrname in base.__dict__ and base is cls:
            return getattr(base, attrname)

    return None



from inspect import getmembers

from sqlalchemy.exc import InvalidRequestError


def get_module_members(modules, predicate):

    members = getmembers(modules, predicate)
    return map(lambda m: m[1], members)


def get_immediate_cls_attr(cls, attrname):
    if not issubclass(cls, object):
        return None

    for base in cls.__mro__:
        if attrname in base.__dict__ and base is cls:
            return getattr(base, attrname)

    return None


def get_package_members(package, predicate, pre_url=''):
    from pkgutil import iter_modules
    from importlib import import_module

    members = {}

    for _, name, ispkg in iter_modules(package.__path__, package.__name__+'.'):
        if ispkg:
            sub_members = get_package_members(
                import_module(name), predicate,
                '/'.join([pre_url, name.split('.')[-1]])
            )
            members.update(sub_members)

        members.setdefault(pre_url, []).extend(
            get_module_members(import_module(name), predicate)
        )

    return members


def common_batch_crud(schema, model, datas):
    """ 批量做增改删操作
    """
    # 删除多余的数据
    schema_instance = schema()
    delete_datas = [data for data in datas if data.get('status') == -1]
    delete_datas, errors = schema_instance.load(delete_datas, many=True)
    if errors:
        raise Exception(errors)

    for delete_data in delete_datas:
        try:
            delete_data.delete()
        except InvalidRequestError:
            pass

    # 新增和修改数据
    modify_datas = [data for data in datas if data.get('status') != -1]
    modify_datas, errors = schema_instance.load(modify_datas, many=True)
    if errors:
        raise Exception(errors)
    [modify_data.save() for modify_data in modify_datas]

    datas = schema(many=True, exclude=model.column_filter).dump(modify_datas)
    return datas



import os
import click


@click.command()
@click.option('--sql', default=False,
              help=("Don't emit SQL to database - dump to standard "
                    "output instead"))
@click.pass_context
def upgrade(ctx, sql=False):
    """ Upgrade database data and strcuct
    """
    from seed.services.app import SeedHttpServer
    from seed.runner.setting import discover_configs

    _, config_file, _ = discover_configs()
    SeedHttpServer(
        host='127.0.0.1',
        port='5000',
        workers=1,
        config_file=config_file
    ).upgrade(sql)



class DataModel(object):
    def __init__(self, db, sql, query, dimensions, indexs):
        self.db = db
        self.sql = sql
        self.query = query
        self.dimensions = dimensions
        self.indexs = indexs
        self.gen_sql()

    def gen_sql(self):
        """ 用维度和指标把SQL包一层
        """
        select_fileds = [item['dimension'] for item in self.dimensions]\
            + ['sum('+item['index']+') as ' + item['index'] for item in self.indexs]
        group_fileds = [item['dimension'] for item in self.dimensions]
        self.sql = """
            SELECT
                {select_fileds}
            FROM (
                {origin_sql}
            ) a
            GROUP BY
                {group_fileds}
        """.format(
            select_fileds=','.join(select_fileds),
            origin_sql=self.sql,
            group_fileds=','.join(group_fileds)
        )

    def format_sql(self):

        for key, value in self.query.items():
            if isinstance(value, list):
                self.query[key] = '(%s)' % ', '.join([str(item) for item in value])

        return self.sql.format(**self.query)

    def query_data(self):
        sql = self.format_sql()
        print(sql)
        return self.db.query(sql), sql



from seed.schema.base import BaseSchema
from seed.api.endpoints._base import RestfulBaseView
from seed.models.role import Role as RoleModel
from seed.utils.auth import api_require_admin


class RoleSchema(BaseSchema):
    class Meta:
        model = RoleModel
        include_fk = True


class Role(RestfulBaseView):
    """ role
    """
    model_class = RoleModel
    schema_class = RoleSchema

    decorators = [api_require_admin]


import smtplib
from email.mime.text import MIMEText
from email.header import Header
from email.utils import formatdate


class Email(object):
    def __init__(self, configs):
        self.mail_backend = configs['MAIL_BACKEND']
        self.mail_host = configs['MAIL_HOST']
        self.mail_port = configs['MAIL_PORT']
        self.mail_user = configs['MAIL_USER']
        self.mail_password = configs['MAIL_PASSWORD']
        self.mail_use_tls = configs['MAIL_USE_TLS']
        self.mail_from = configs['MAIL_FROM']

    def send_mail(self, to_mail_list, title, message, message_type='plain', cc_mail_list=[]):
        to_mails = '; '.join(to_mail_list)
        cc_mails = '; '.join(cc_mail_list)

        mime_text = MIMEText(message.encode('utf-8'), message_type, 'utf-8')
        mime_text['Subject'] = Header(title, 'utf-8')
        mime_text['From'] = self.mail_from
        mime_text['To'] = to_mails
        mime_text['Cc'] = cc_mails
        mime_text['Date'] = formatdate()

        smtp = smtplib.SMTP(self.mail_host, self.mail_port)
        smtp.ehlo()
        smtp.starttls()
        smtp.ehlo()
        smtp.login(self.mail_user, self.mail_password)

        smtp.sendmail(self.mail_from, to_mail_list+cc_mail_list, mime_text.as_string())

        smtp.close()



import json
from flask import request, g
from sqlalchemy import and_

from seed.schema.base import BaseSchema
from seed.api.endpoints._base import RestfulBaseView, HttpMethods
from seed.models.rolemenu import RoleMenu as RoleMenuModel
from seed.models.menu import Menu as MenuModel
from seed.utils.auth import api_require_admin


class RoleMenuSchema(BaseSchema):
    class Meta:
        model = RoleMenuModel
        include_fk = True


class RoleMenu(RestfulBaseView):
    """ 角色菜单设置
    """
    model_class = RoleMenuModel
    schema_class = RoleMenuSchema

    decorators = [api_require_admin]

    access_methods = [HttpMethods.GET, HttpMethods.PUT]

    def get(self, role_id):
        """ GET
        """
        role_menu = self._get_role_menu(role_id)

        menus = self._encode_menus(role_menu)

        return self.response_json(self.HttpErrorCode.SUCCESS, data=menus)

    def put(self, role_id):
        request_json = request.get_json()
        menus = request_json['menu']
        self._decode_menus(menus, role_id)
        return self.response_json(self.HttpErrorCode.SUCCESS)

    def _get_role_menu(self, model_id):
        role_datas = self.session.query(RoleMenuModel)\
            .filter(RoleMenuModel.role_id == model_id, RoleMenuModel.bussiness_id == g.bussiness_id).all()
        role_data_map = {role_data.menu_id: role_data for role_data in role_datas}

        menu_datas = self.session.query(MenuModel)\
            .filter(MenuModel.bussiness_id == g.bussiness_id).all()

        menu_datas_with_permission = {}

        for menu_data in menu_datas:
            menu_data = menu_data.row2dict()
            role_data = role_data_map[menu_data['id']] if menu_data['id'] in role_data_map else None

            menu_data['role_permission'] = role_data.role_permission if role_data else False
            menu_data['menu_id'] = menu_data['id']
            if role_data:
                menu_data['id'] = role_data.id

            menu_datas_with_permission['-'.join([str(menu_data['parent_id']), str(menu_data['left_id'])])] = menu_data

        return menu_datas_with_permission

    def _encode_menus(self, menu_data):
        menus = {'id': 0}
        middle_menu = [menus]
        while middle_menu:
            current_menu = middle_menu.pop()
            parent_id, left_id = current_menu['id'], 0

            while True:
                current_key = '-'.join([str(parent_id), str(left_id)])
                if current_key not in menu_data:
                    break
                current_menu.setdefault('sub_menus', []).append(menu_data[current_key])
                middle_menu.append(menu_data[current_key])
                left_id = menu_data[current_key]['id']

        return menus.get('sub_menus', [])

    def _decode_menus(self, menus, role_id, parent_id=0, left_id=0):
        if not menus:
            return

        for menu in menus:
            # 更新或插入新的菜单
            # 获取到菜单对应的ID
            # if menu.get('role_permission', False):
            self._insert_or_update_menu(menu, role_id, menu.get('role_permission', False))
            current_id = menu['menu_id']
            self._decode_menus(menu.get('sub_menus', []), role_id=role_id, parent_id=current_id, left_id=0)

    def _insert_or_update_menu(self, menu, role_id, role_permission):
        role_menu = {
            "role_id": role_id,
            "menu_id": menu['menu_id'],
            "role_permission": role_permission
        }
        if 'id' in menu:
            role_menu['id'] = menu['id']
        schema = self.schema_class()
        datas, errors = schema.load(role_menu)
        datas.save()
        return datas.id



import os
import click

from seed.utils.imports import import_string

version_string = '0.1'


@click.group()
@click.option(
    '--config',
    default='',
    envvar='SEED_CONF',
    help='Path to config file',
    metavar='PATH'
)
@click.version_option(version=version_string)
@click.pass_context
def cli(ctx, config):
    if config:
        os.environ['SEED_CONF'] = config
    else:
        os.environ['SEED_CONF'] = '~/.seed'


cmds = [
    'seed.runner.commands.run.run',
    'seed.runner.commands.init.init',
    'seed.runner.commands.upgrade.upgrade',
]

for cmd in cmds:
    cli.add_command(import_string(cmd))


def main():
    cli()



import json
import logging
import functools
from datetime import datetime

import requests
from flask import current_app, jsonify, g, request

from seed.utils.response import response, HttpErrorCode
from seed.models.account import Account
from seed.models.bmanager import BManager
from seed.cache import DefaultCache
from seed.cache.session import SessionCache
from seed.cache.user_bussiness import UserBussinessCache


class APIRequireRole(object):
    roles = {
        "new": 0,
        "user": 1,
        "admin": 2,
        "super_admin": 3
    }

    def __init__(self, role):
        self.role = role

    def __call__(self, method):
        @functools.wraps(method)
        def wrapper(*args, **kwargs):
            if not g.user:
                # 未登录
                return jsonify(response(HttpErrorCode.UNAUTHORIZED))

            if g.user.status == -1:
                # 废弃账号
                return jsonify((response(HttpErrorCode.FORBIDDEN)))

            if not self.role or g.user.id == 1:
                return method(*args, **kwargs)

            if self.roles[g.user.role] < self.roles[self.role]:
                # 没有足够的权限
                return jsonify(response(HttpErrorCode.FORBIDDEN))

            return method(*args, **kwargs)

        return wrapper


api_require_login = APIRequireRole(None)
api_require_new = APIRequireRole("new")
api_require_user = APIRequireRole("user")
api_require_admin = APIRequireRole("admin")
api_require_super_admin = APIRequireRole("super_admin")


class RequireRole(object):
    roles = {
        "new": 0,
        "user": 1,
        "admin": 2,
        "super_admin": 3
    }

    def __init__(self, role):
        self.role = role

    def __call__(self):
        if not g.user:
            # 未登录
            return False

        if g.user.status == -1:
            # 废弃账号
            return False

        if not self.role or g.user.id == 1:
            return True

        if self.roles[g.user.role] < self.roles[self.role]:
            # 没有足够的权限
            return False

        return True


require_login = RequireRole(None)
require_new = RequireRole("new")
require_user = RequireRole("user")
require_admin = RequireRole("admin")
require_super_admin = RequireRole("super_admin")


class BaseAuth(object):
    """ 验证基础类
    """
    def get_current_user(self):
        """ 获取当前用户
        """
        raise NotImplementedError("Not Implemented")

    def login_user(self, user):
        """ 登录用户
        """
        raise NotImplementedError("Not Implemented")

    def logout_user(self):
        """ 登出用户
        """
        raise NotImplementedError("Not Implemented")

    def debbuger_user(self):
        if not require_super_admin() or 'debugger' not in request.args:
            return g.user

        user = Account.query.filter_by(id=int(request.args['debugger'])).first()

        if user:
            bussiness_id = UserBussinessCache().get(user.id) or 1
            if self._is_bussiness_admin(user.id, bussiness=bussiness_id):
                user.role = 'admin'

        return user if user else None

    def _is_bussiness_admin(self, uid, bussiness=1):
        roles = BManager.query.filter_by(bussiness_id=bussiness, user_id=uid).all()
        return True if roles else False


class SessionAuth(BaseAuth):
    """ Session登录权限类
    """
    def get_current_user(self):
        session_token = request.cookies.get('session_token', '')
        user_id = SessionCache().get_user_id_by_token(session_token)
        if not user_id:
            return None

        user = Account.query.filter_by(id=user_id).first()
        if user:
            bussiness_id = UserBussinessCache().get(user.id) or 1
            # TODO 需要修复db.model自动保存的问题
            if self._is_bussiness_admin(user.id, bussiness=bussiness_id) and user.role != 'super_admin':
                user.role = 'admin'

            user.role = 'super_admin' if user.id == 1 else user.role

        return user


class SSOAuth(BaseAuth):
    """ SSO权限校验登录类
    """
    def get_current_user(self):
        # 获取当前用户信息在redis中存储的信息
        uid = request.cookies.get('admin_uid', None)
        uid_key = request.cookies.get('admin_key', None)

        if not (uid and uid_key):
            return None

        # 判断信息的有效性
        today = datetime.strftime(datetime.today(), '%Y-%m-%d')
        login_at = self._get_login_cache(uid)
        if today != login_at:
            # 去SSO中校验用户的有效性
            user_info = self._sso_verification(uid, uid_key)
            if not user_info:
                return None

            # 创建新用户 或者 获取用户的user_id
            user = self._get_user_id(user_info)

            self._account_valid(user)

            # 存储用户信息到Redis中
            self._set_login_cache(uid, today)
        else:
            user = Account.query.filter_by(sso_id=int(uid)).first()

        if user:
            bussiness_id = UserBussinessCache().get(user.id) or 1
            if self._is_bussiness_admin(user.id, bussiness=bussiness_id) and user.role != 'super_admin':
                user.role = 'admin'

            user.role = 'super_admin' if user.id == 1 else user.role

        return user

    def logout_user(self):
        session = []

    def cache(self):
        return DefaultCache(current_app.cache)

    def _account_valid(self, user):
        if user.status == -1:
            raise Exception("账号已经被注销")

    def _get_user_id(self, user_info):
        account = Account.query.filter_by(sso_id=int(user_info['id'])).first()
        if not account:
            # 创建信息用户
            account = Account(
                sso_id=int(user_info['id']),
                account=user_info['username'],
                email=user_info['email'],
                avatar='http://oahead-static.17c.cn/oahead/' + str(int(user_info['code'])) + '.png',
                name=user_info['cname'],
                role='user',
                status=1
            )
        else:
            account.login_at = datetime.utcnow()
        account.save()
        return account

    def _sso_verification(self, uid, uid_key):
        check_sso_user_url = "http://sso.ifere.com:8871/api?do=getInfo&uid=%s&key=%s&appid=1172" % (uid, uid_key)

        try:
            data_str = requests.get(check_sso_user_url, timeout=5)
            ret = json.loads(data_str.text)

            if "ret" not in ret or '1' != str(ret['ret']):
                logging.warning('user is invalid uid:%s uid_key:%s' % (uid, uid_key))
                return []

            return ret
        except Exception as e:

            logging.warning('error: %s' % e)
            return []

    def _set_login_cache(self, uid, login_at):
        self.cache().set(':'.join([uid, 'sso_login']), login_at)

    def _get_login_cache(self, uid):
        return self.cache().get(':'.join([uid, 'sso_login']))





class ModelProxyCache(dict):
    def __missing__(self, path):
        if '.' not in path:
            return __import__(path)

        module_name, class_name = path.rsplit('.', 1)

        module = __import__(module_name, {}, {}, [class_name])
        handler = getattr(module, class_name)

        self[path] = handler
        return handler


_cache = ModelProxyCache()


def import_string(path):
    """ Path must be models.path.ClassName
    """
    result = _cache[path]
    return result



import json

from flask import request, g
from marshmallow import pre_load, post_dump

from seed.schema.base import BaseSchema

from seed.api.endpoints._base import RestfulBaseView

from seed.models.filters import Filters as FiltersModel
from seed.utils.auth import api_require_login


class FilterSchema(BaseSchema):
    class Meta:
        model = FiltersModel
        include_fk = True

    @post_dump(pass_many=True)
    def dumps_index(self, in_data, many):
        if many:
            for data in in_data:
                data['conditions'] = json.loads(data['conditions'])
                data['cascades'] = json.loads(data['cascades'] or '{}')
        else:
            in_data['conditions'] = json.loads(in_data['conditions'])
            in_data['cascades'] = json.loads(in_data['cascades'] or '{}')

        return in_data

    @pre_load(pass_many=True)
    def loads_conditions(self, out_data, many):
        if many:
            for data in out_data:
                data['conditions'] = json.dumps(data['conditions'])
                data['cascades'] = json.dumps(data.get('cascades', {}))
        else:
            out_data['conditions'] = json.dumps(out_data['conditions'])
            out_data['cascades'] = json.dumps(out_data.get('cascades', {}))
        return out_data


class Filters(RestfulBaseView):
    """ 过滤组件
    """
    model_class = FiltersModel
    schema_class = FilterSchema
    decorators = [api_require_login]


import os


DEFAULT_SETTINGS_CONF = 'config.yaml'
DEFAULT_SETTINGS_OVERRIDE = 'seed_conf.py'


def discover_configs():
    """ discover seed config files
    """

    try:
        config = os.environ['SEED_CONF']
    except KeyError:
        config = '~/.seed'

    config = os.path.expanduser(config)

    return (
        config, os.path.join(config, DEFAULT_SETTINGS_OVERRIDE),
        os.path.join(config, DEFAULT_SETTINGS_CONF)
    )


def load_config_template(path, version='default'):
    from pkg_resources import resource_string
    return resource_string('seed', 'data/config/%s.%s' % (path, version)).decode('utf-8')


def generate_settings(dev):
    """ generate default setting file contents
    """

    context = {
        'debug_flag': dev,
    }

    py = load_config_template(DEFAULT_SETTINGS_OVERRIDE, 'default') % context
    yaml = load_config_template(DEFAULT_SETTINGS_CONF, 'default') % context

    return py, yaml



"""
seed.conf.server
~~~~~~~~~~~~~~~~

There setting act as default(base) settings for the seed-provided web-server
"""

import os

ENVRIOMENT = os.environ.get('SEED_ENVRIOMENT', 'production')

IS_DEV = ENVRIOMENT == 'development'

DEBUG = IS_DEV

# Seed logs formatting
LOGGING = {
    'default_level': 'INFO',
    'version': 1,
    'disable_existing_loggers': True,
    'handlers': {
        'null': {
            'class': 'logging.NullHandler',
        },
        'console': {
            'class': 'logging.StreamHandler'
        },
    },
}


from flask import current_app
from seed.cache.active_account import ActiveAccountCache


class ResetPasswordCache(ActiveAccountCache):
    def __init__(self):
        self.redis_client = current_app.cache
        self.middle_key = 'reset_password'



import os
from pathlib import Path

import redis
import flask_migrate
from flask import Flask, g
from flask_cors import CORS

from seed.runner.setting import discover_configs

from seed.models import db, migrate, session, ma
from seed.api.urls import register_api
from seed.utils.auth import SSOAuth, SessionAuth
from seed.cache.user_bussiness import UserBussinessCache
from seed.models.init import init_databases, init_analogdata, is_new_databases


class SeedHttpServer(object):
    def __init__(
        self, host, port, workers, config_file, extra_options=None
    ):
        self.host, self.port = host, port
        self.workers = workers

        self.app = self.create_app(config_file)

        self.register_cors()
        self.register_databases()
        self.register_cache()
        self.register_api()
        self.register_hook()

    def create_app(self, config_file):
        static_folder_path = os.path.join(
            Path(os.path.dirname(os.path.realpath(__file__))).parent,
            'static',
            'static'
        )
        template_folder_path = os.path.join(
            Path(os.path.dirname(os.path.realpath(__file__))).parent,
            'static'
        )
        app = Flask(
            __name__,
            static_url_path='/static',
            static_folder=static_folder_path,
            template_folder=template_folder_path
        )

        app.config.from_pyfile(config_file)

        return app

    def register_databases(self):
        db.init_app(self.app)
        migrate.init_app(app=self.app, db=db)
        ma.init_app(self.app)

        with self.app.app_context():
            session.configure(bind=db.engine)

    def register_cache(self):
        redis_pool = redis.ConnectionPool.from_url(
            self.app.config['REDIS_URL'],
            decode_components=True
        )
        self.app.cache = redis.Redis(connection_pool=redis_pool)

    def register_api(self):
        register_api(self.app)

        from seed.api import front
        self.app.register_blueprint(front.bp, url_prefix='')

    def register_hook(self):
        """ 注册Hook
        """
        @self.app.before_request
        def login_user():

            if self.app.config['AUTH_TYPE'] == 'SSO':
                auth = SSOAuth()
            else:
                auth = SessionAuth()

            g.user = auth.get_current_user()

            # debugger
            g.user = auth.debbuger_user()

            if g.user:
                g.bussiness_id = UserBussinessCache().get(g.user.id) or -1
            else:
                g.bussiness_id = -1

    def register_cors(self):
        CORS(self.app, supports_credentials=True)

    def run(self):
        self.app.run(self.host, self.port)

    def upgrade(self, sql):
        print('数据库开始初始化')
        with self.app.app_context():
            migrate_directory = self.app.extensions['migrate'].directory

            migrate_path, _, _ = discover_configs()
            migrate_path = os.path.join(migrate_path, migrate_directory)

            if not os.path.exists(os.path.join(migrate_path, 'alembic.ini')):
                # 判断是不是第一次初始化数据库
                is_empty = input('第一次初始化数据库，请确保你的数据库是空的。Y/N? ')
                if is_empty != 'Y':
                    return

                flask_migrate.init(migrate_path)

            flask_migrate.migrate(migrate_path, sql=sql)
            flask_migrate.upgrade(migrate_path, sql=sql)

            if is_new_databases():
                # 写入默认数据
                print('初始化模拟数据')
                init_analogdata()
                print('初始化默认业务模块')
                init_databases()

        print("数据库升级完成")



from flask import request

from seed.api.endpoints._base import RestfulBaseView, HttpMethods
from seed.cache.active_account import ActiveAccountCache
from seed.models.account import Account


class ActiveAccount(RestfulBaseView):
    """ 激活账户
    """
    url = 'active_account'
    access_methods = [HttpMethods.GET]

    def get(self):
        active_token = request.args.get('active_token')
        if not active_token:
            return self.response_json(
                self.HttpErrorCode.PARAMS_VALID_ERROR,
                msg='激活token参数缺失'
            )

        user_id = ActiveAccountCache().get_user_by_active_token(active_token)
        if not user_id:
            return self.response_json(
                self.HttpErrorCode.PARAMS_VALID_ERROR,
                msg='激活token已经失效'
            )

        account = self.session.query(Account).filter_by(id=user_id).first()
        if not account:
            return self.response_json(
                self.HttpErrorCode.PARAMS_VALID_ERROR,
                msg='激活账户不存在'
            )

        account.role = 'user'
        account.status = 1
        account.save()

        return self.response_json(self.HttpErrorCode.SUCCESS)



from ._base import db, BussinessModel

__all__ = ["RoleMenu",]


class RoleMenu(BussinessModel):
    role_id = db.Column(db.Integer)
    menu_id = db.Column(db.Integer)

    role_permission = db.Column(db.Boolean, default=True)


import random
import string

from flask import current_app

from seed.cache.redis import RedisCache


class SessionCache(RedisCache):
    def __init__(self):
        self.redis_client = current_app.cache
        self.middle_key = 'session'

    def make_key(self, key):
        key = ':'.join([self.middle_key, str(key)])
        return super(SessionCache, self).make_key(key)

    def create_session(self, user_id):
        token = ''.join([random.choice(string.ascii_uppercase + string.digits) for key in range(32)])
        self.set(token, user_id, timeout=24 * 60 * 60)
        return token

    def get_user_id_by_token(self, token):
        return self.get(token)



from flask import request, g

from seed.schema.base import BaseSchema
from seed.api.endpoints._base import RestfulBaseView, HttpMethods
from seed.models.menu import Menu as MenuModel
from seed.utils.auth import api_require_admin


class MenuSchema(BaseSchema):
    class Meta:
        model = MenuModel


class Menu(RestfulBaseView):
    """ menu
    """
    model_class = MenuModel
    schema_class = MenuSchema

    decorators = [api_require_admin]

    access_methods = [HttpMethods.GET, HttpMethods.POST]

    def get(self):
        """ GET
        """
        query_session = self.session.query(self.model_class).filter(self.model_class.bussiness_id==g.bussiness_id)
        menu_data = query_session.all()
        menus = self._encode_menus(menu_data)
        return self.response_json(self.HttpErrorCode.SUCCESS, data=menus)

    def post(self):
        """ 更新菜单结构
        """
        menus = request.get_json()
        self._decode_menus(menus)

        query_session = self.session.query(self.model_class).filter(self.model_class.bussiness_id==g.bussiness_id)
        menu_data = query_session.all()
        menus = self._encode_menus(menu_data)

        return self.response_json(self.HttpErrorCode.SUCCESS, data=menus)

    def _encode_menus(self, menu_data):
        menu_data = {'-'.join([str(row.parent_id), str(row.left_id)]): row.row2dict() for row in menu_data}

        menus = {'id': 0}
        middle_menu = [menus]
        while middle_menu:
            current_menu = middle_menu.pop()
            parent_id, left_id = current_menu['id'], 0

            while True:
                current_key = '-'.join([str(parent_id), str(left_id)])
                if current_key not in menu_data:
                    break
                current_menu.setdefault('sub_menus', []).append(menu_data[current_key])
                middle_menu.append(menu_data[current_key])
                left_id = menu_data[current_key]['id']

        return menus.get('sub_menus', [])

    def _decode_menus(self, menus, parent_id=0, left_id=0):
        if not menus:
            return

        for menu in menus:
            # 更新或插入新的菜单
            # 获取到菜单对应的ID
            left_id = current_id = self._update_menu_item(menu, parent_id, left_id)
            self._decode_menus(menu.get('sub_menus', []), parent_id=current_id, left_id=0)

    def _update_menu_item(self, menu, parent_id, left_id):
        menu.update({
            'parent_id': parent_id,
            'left_id': left_id
        })
        schema = self.schema_class()
        datas, errors = schema.load(menu)
        if menu.get('status', 0) == -1:
            datas.delete()
        else:
            datas.save()
        return datas.id



from seed.models._base import db, PageModel

__all__ = ['Panels']


class Panels(PageModel):
    """ data panels model
    """
    name = db.Column(
        db.Text, nullable=False,
        comment='Panel name'
    )

    desc = db.Column(
        db.Text, nullable=False,
        comment='Panel description'
    )

    x = db.Column(
        db.Float, nullable=False,
        comment='The panel x coordinate'
    )

    y = db.Column(
        db.Float, nullable=False,
        comment='The panel y coordinate'
    )

    w = db.Column(
        db.Float, nullable=False,
        comment='The panel width'
    )

    h = db.Column(
        db.Float, nullable=False,
        comment='The panel height'
    )

    db_source = db.Column(
        db.Integer, nullable=False,
        comment='Panel sql query database id'
    )

    charttype = db.Column(
        db.Text, nullable=False,
        comment='Panel data chart type'
    )

    sql = db.Column(
        db.Text, nullable=False,
        comment='Panel sql string'
    )

    indexs = db.Column(
        db.Text, nullable=False,
        comment='Data index json configure'
    )

    dimensions = db.Column(
        db.Text, nullable=False,
        comment='Data dimensions configure'
    )

    sort = db.Column(
        db.Integer, nullable=False,
        comment='The panel sort'
    )


from flask import request, g
from sqlalchemy.exc import InvalidRequestError

from seed.api.endpoints._base import RestfulBaseView, HttpMethods

from seed.models.filters import Filters as FiltersModel
from seed.api.endpoints.filters import FilterSchema
from seed.models.panels import Panels as PanelsModel
from seed.api.endpoints.panels import PanelSchema

from seed.utils.helper import common_batch_crud


class Pages(RestfulBaseView):
    """ 页面
    """
    access_methods = [HttpMethods.GET, HttpMethods.PUT]

    def get(self, page_id):
        global_filters = self._get_global_filters(page_id)
        panels = self._get_panels(page_id)

        data = {
            "global_filters": global_filters,
            "panels": panels
        }
        return self.response_json(self.HttpErrorCode.SUCCESS, data=data)

    def put(self, page_id):
        input_data = request.get_json()

        global_filters, panels = input_data['global_filters'], input_data['panels']
        for global_filter in global_filters:
            global_filter['bussiness_id'] = g.bussiness_id
        global_filters = common_batch_crud(FilterSchema, FiltersModel, global_filters)

        panels = self._panels_batch_crud(PanelSchema, panels)

        data = {'global_filters': global_filters, 'panels': panels}

        return self.response_json(self.HttpErrorCode.SUCCESS, data=data)

    def _get_global_filters(self, page_id):
        """ 获取页面全局过滤组件数据
        """
        query_session = self.session.query(FiltersModel)

        filters = query_session.filter_by(page_id=page_id, dtype='page').all()
        filters, errors = FilterSchema(many=True, exclude=FiltersModel.column_filter).dump(filters)

        return filters

    def _get_panels(self, page_id):
        """ 获取过滤组件数据
        """
        panel_query_session = self.session.query(PanelsModel)
        panels = panel_query_session.filter_by(page_id=page_id).order_by(PanelsModel.sort).all()
        panels, errors = PanelSchema(many=True, exclude=PanelsModel.column_filter).dump(panels)

        filter_query_session = self.session.query(FiltersModel)
        filters = filter_query_session.filter_by(page_id=page_id, dtype='model').all()
        filters, errors = FilterSchema(many=True, exclude=FiltersModel.column_filter).dump(filters)

        filters_pid_map = {}
        for filter in filters:
            filters_pid_map.setdefault(filter['belong_id'], []).append(filter)

        for panel in panels:
            panel['filters'] = filters_pid_map.get(panel['id'], [])

        return panels

    def _panels_batch_crud(self, panel_schema, panels):
        schema_instance = panel_schema()

        # 删除panels
        saved_panels = []
        for panel in panels:
            one_panel, errors = schema_instance.load(panel)
            if errors:
                raise Exception(errors)

            if panel.get('status') == -1:
                # 删除对应的panel
                try:
                    one_panel.delete()
                except InvalidRequestError:
                    pass

            else:
                # 新增和修改对应的panel
                one_panel.save()
                panel_filters = panel.get('filters', [])

                for filter in panel_filters:
                    filter['belong_id'] = one_panel.id
                    filter['bussiness_id'] = g.bussiness_id
                    filter['page_id'] = one_panel.page_id

                panel_filters = common_batch_crud(FilterSchema, FiltersModel, panel_filters)
                panel, errors = panel_schema(exclude=PanelsModel.column_filter).dump(one_panel)
                panel['filters'] = panel_filters
                saved_panels.append(panel)
        return saved_panels



from . import db, BussinessModel

__all__ = ['Role', ]


class Role(BussinessModel):
    role = db.Column(db.Text, default='new')

    @classmethod
    def get_roles(cls, bussiness_id=None):
        query_session = cls.query
        if bussiness_id:
            query_session = query_session.filter_by(bussiness_id=bussiness_id)

        return query_session.all()



import psycopg2

from seed.drives.base import BaseDrive, Row

# 解决 pg TypeError: Decimal('30737') is not JSON serializable， 的bug
DEC2FLOAT = psycopg2.extensions.new_type(
    psycopg2.extensions.DECIMAL.values,
    'DEC2FLOAT',
    lambda value, curs: float(value) if value is not None else None)
psycopg2.extensions.register_type(DEC2FLOAT)
# 解决DataTime is not Json serializable的bug
DATE2STR = psycopg2.extensions.new_type(
    psycopg2.extensions.PYDATE.values,
    'DATE2STR',
    lambda value, curs: value.split('.')[0] if value is not None else None)
psycopg2.extensions.register_type(DATE2STR)
DATETIME2STR = psycopg2.extensions.new_type(
    psycopg2.extensions.PYDATETIME.values,
    'DATETIME2STR',
    lambda value, curs: value.split('.')[0] if value is not None else None)
psycopg2.extensions.register_type(DATETIME2STR)


class PostgreSQL(BaseDrive):
    def query(self, sql, params=[], retry_count=1):
        """ 查询SQL语句
        """
        self._raise_retry_count(retry_count)

        cursor = self._gen_cursor()
        try:
            query_datas = self._query(cursor, sql, params)
            return query_datas
        except (psycopg2.OperationalError, psycopg2.DatabaseError, psycopg2.InterfaceError) as e:
            # 可能数据库会主动断开连接
            # 尝试重新连接, 并重试
            if not retry_count:
                raise

            self._connect()
            return self.query(sql, params=params, retry_count=retry_count-1)
        finally:
            cursor.close()

        return []

    def _query(self, cursor, sql, params):
        self._execute(cursor, sql, params=params)
        columns_name = [d[0] for d in cursor.description]
        query_datas = [Row(zip(columns_name, row)) for row in cursor]

        return query_datas

    def _execute(self, cursor, sql, params=[]):
        """ 执行SQL
        """
        try:
            cursor.execute(sql, params)
            self._commit()
        except psycopg2.ProgrammingError as e:
            # syntax error
            self._rollback()
            raise Exception(str(e))
        except psycopg2.extensions.QueryCanceledError as e:
            raise psycopg2.QueryCanceledError(str(e))
        except (psycopg2.OperationalError, psycopg2.DatabaseError, psycopg2.InterfaceError) as e:
            # server closed the connection unexpectedly
            raise
        except Exception as e:
            self._rollback()
            raise

    def _get_connection(self):
        if self.alive():
            self._connect()

    def _commit(self):
        """ 完成SQL执行
        """
        self.conn.commit()

    def _rollback(self):
        """ 回滚SQL
        """
        self.conn.rollback()

    def _connect(self):
        try:
            self.conn = psycopg2.connect(
                dbname=self.name,
                user=self.user,
                password=self.password,
                host=self.ip,
                port=self.port,
                options='-c statement_timeout=60s',
            )
        except Exception as e:
            raise

    def _gen_cursor(self):
        cursor = self.conn.cursor()
        return cursor

    def alive(self):
        """ 测试连接是否还存在
        """
        if self.conn:
            return True if self.conn.closed == 0 else False

        return False

    def close(self):
        """ 关闭连接
        """
        self.conn.close()



import json
import collections


class MiddleData(object):
    """ 原始数据格式 转 中间数据格式
    row = [
        {'dau': 100, 'dsu': 200, 'fdate': '2017-01-01'},
        {'dau': 100, 'dsu': 200, 'fdate': '2017-01-02'},
    ]

    reuslt = {
        json.loads({'fdata': '2017-01-01'}): {'dau': 100, 'dsu': 200},
        json.loads({'fdata': '2017-01-02'}): {'dau': 200, 'dsu': 100},
    }
    """

    def __init__(self, source_data, dimensions, indexs):
        self.source_data = source_data
        self.dimensions = [item['dimension'] for item in dimensions]
        self.indexs = [item['index'] for item in indexs]

    def convert(self):
        middle_datas = {}

        for row in self.source_data:

            # 根据dimensions构造当前数据项的key
            row_keys = collections.OrderedDict()
            for dimension in self.dimensions:
                row_keys[dimension] = row[dimension]

            for index in self.indexs:
                middle_key = json.dumps(row_keys)
                if middle_datas.get(middle_key, {}).get(index) is None:
                    middle_datas.setdefault(middle_key, {})[index] = row.get(index, None)

        return middle_datas



from flask import current_app

from seed.cache.redis import RedisCache


class UserBussinessCache(RedisCache):
    def __init__(self):
        self.redis_client = current_app.cache
        self.middle_key = 'user_bussiness'

    def make_key(self, key):
        key = ':'.join([self.middle_key, str(key)])
        return super(UserBussinessCache, self).make_key(key)


from flask import g
from seed.schema.base import BaseSchema
from seed.api.endpoints._base import RestfulBaseView, HttpMethods

from seed.models.account import Account as AccountModel
from seed.utils.auth import api_require_user, require_super_admin


class AccountSchema(BaseSchema):
    class Meta:
        model = AccountModel


class Account(RestfulBaseView):
    """ Account
    """
    model_class = AccountModel
    schema_class = AccountSchema

    access_methods = [HttpMethods.GET, HttpMethods.PUT, HttpMethods.DELETE]

    decorators = [api_require_user]

    def put(self, model_id=None):
        if not require_super_admin() and g.user.id != int(model_id): 
            return self.response_json(self.HttpErrorCode.FORBIDDEN)

        return super(Account, self).put(model_id=model_id)

    def delete(self, model_id):
        if not require_super_admin():
            return self.response_json(self.HttpErrorCode.FORBIDDEN)

        return super(Account, self).delete(model_id=model_id)



from . import db, BaseModel

__all__ = ['Databases']


class Databases(BaseModel):
    """ databases configure
    """
    bussiness_id = db.Column(db.Integer, db.ForeignKey('bussiness.id'))
    dtype = db.Column(
        db.Text, nullable=False,
        comment='Database type, like mysql, postgresql'
    )
    name = db.Column(
        db.Text, nullable=False,
        comment='Database name'
    )
    ip = db.Column(
        db.Text, nullable=False,
        comment='Database IP address'
    )
    port = db.Column(
        db.Text, nullable=False,
        comment='Database port'
    )
    user = db.Column(
        db.Text, nullable=False,
        comment='Database user login'
    )
    password = db.Column(
        db.Text, nullable=False,
        comment='Database user password'
    )



from threading import Thread

class BaseCache(Thread):
    prefix = "seed"

    def __init__(self, prefix=None):
        if not prefix:
            self.prefix = prefix

    def make_key(self, key):
        return '{}:{}'.format(
            self.prefix,
            key
        )

    def get(self, key):
        raise NotImplementedError

    def set(self, key, value, timeout):
        raise NotImplementedError

    def delete(self, key):
        raise NotImplementedError



from datetime import datetime
# from flask.ext.sqlalchemy import Column
from ._base import db, BaseModel


__all__ = ['Account',]


class Account(BaseModel):
    column_filter = ['updated', 'password']

    sso_id = db.Column(db.Integer, default=-1)  # SSO_ID

    account = db.Column(db.String(40), unique=True, index=True, nullable=False)
    password = db.Column(db.String(256))
    email = db.Column(db.String(40), nullable=False)

    avatar = db.Column(db.Text)
    name = db.Column(db.String(40), nullable=False)
    sex = db.Column(db.String(20), nullable=False, default='male')
    depart_id = db.Column(db.Integer, default=-1)  # -1为未知

    role = db.Column(db.String(20), default='new')
    status = db.Column(db.Integer, default=0)  # -1:不可用 0: 未激活 1: 正常使用

    login_at = db.Column(
        db.DateTime, default=datetime.utcnow()
    )


from flask import request, g

from seed.schema.base import BaseSchema
# from seed.api.endpoints._base import RestfulBaseView
from seed.api.endpoints.buser import Buser
from seed.models.databases import Databases as DatabasesModel
from seed.utils.auth import api_require_admin
from seed.utils.helper import common_batch_crud


class DatabasesSchema(BaseSchema):
    class Meta:
        model = DatabasesModel
        include_fk = True


class Databases(Buser):
    """ 数据库设置
    """
    model_class = DatabasesModel
    schema_class = DatabasesSchema
    decorators = [api_require_admin]

    def get(self, bussiness_id):
        """ 通过业务ID得到数据库列表
        """
        query_session = self.session.query(self.model_class)
        datas = query_session.filter(self.model_class.bussiness_id == bussiness_id).all()
        datas = [row.row2dict() for row in datas] if datas else []
        return self.response_json(self.HttpErrorCode.SUCCESS, data=datas)

    def put(self, bussiness_id):
        input_json = request.get_json()

        for database in input_json:
            database['bussiness_id'] = bussiness_id

        databases = common_batch_crud(DatabasesSchema, DatabasesModel, input_json)

        return self.response_json(self.HttpErrorCode.SUCCESS, data=databases)


import os
import click

from seed.runner.commands.run import Address


@click.command()
@click.option(
    '--dev', default=False, is_flag=True, help='Use settings more conducive to local development'
)
@click.argument('directory', required=False)
@click.pass_context
def init(ctx, dev, directory):
    """ Initialize new configuration directory.
    """
    from seed.runner.setting import discover_configs, generate_settings

    if directory:
        os.environ['SEED_CONF'] = directory

    directory, py, yaml = discover_configs()

    if directory and not os.path.exists(directory):
        os.makedirs(directory)

    py_contents, yaml_contents = generate_settings(dev)

    if os.path.isfile(yaml):
        click.confirm(
            "File already exists at '%s', overwrite?" % click.format_filename(yaml),
            abort=True
        )

    with click.open_file(yaml, 'w') as fp:
        fp.write(yaml_contents)

    if os.path.isfile(py):
        click.confirm(
            "File already exists at '%s', overwrite?" % click.format_filename(py),
            abort=True
        )

    with click.open_file(py, 'w') as fp:
        fp.write(py_contents)



from ._base import db, BaseModel

__all__ = ["BManager", ]


class BManager(BaseModel):
    """ 业务管理员模型
    """
    bussiness_id = db.Column(db.Integer, db.ForeignKey('bussiness.id'))
    user_id = db.Column(db.Integer, nullable=False)



from seed.models._base import db, PageModel

__all__ = ['Filters']


class Filters(PageModel):
    """ Filter model class
    """
    belong_id = db.Column(
        db.Integer,
        comment='Filter belong which model'
    )
    dtype = db.Column(
        db.Text, nullable=False,
        comment='Filter belong type, page or model'
    )
    stype = db.Column(
        db.Text, nullable=False,
        default='single', comment='Single or mulitple choice'
    )
    ename = db.Column(
        db.Text, nullable=False,
        comment='Field english name'
    )
    cname = db.Column(
        db.Text, nullable=False,
        comment='Field chinese name'
    )
    cascades = db.Column(
        db.Text, default=0,
        comment='The Filter field cascade id if it exists'
    )
    db_source = db.Column(
        db.Integer,
        comment='The filter data query database id only for sql'
    )
    condition_type = db.Column(
        db.Text, nullable=False,
        comment='The Filter condition setting type, manual or sql'
    )
    conditions = db.Column(
        db.Text, nullable=False,
        comment='The Filter conditions, it will save as json string'
    )
    sort = db.Column(
        db.Integer, default=0,
        comment='The filter sort'
    )



import bcrypt
from flask import request

from seed.api.endpoints._base import RestfulBaseView, HttpMethods
from seed.cache.active_account import ActiveAccountCache
from seed.schema.base import BaseSchema
from seed.models.account import Account
from seed.utils.mail import send_active_email


class AccountSchema(BaseSchema):
    """  账户
    """
    class Meta:
        model = Account


class Register(RestfulBaseView):
    """ 注册
    """
    access_methods = [HttpMethods.POST]

    def post(self):
        """ POST
        """
        input_json = request.get_json()

        if input_json['password'] != input_json['confirm_password']:
            return self.response_json(
                self.HttpErrorCode.PARAMS_VALID_ERROR,
                msg='密码和确认密码不一致'
            )

        if 'active_url' not in input_json:
            return self.response_json(
                self.HttpErrorCode.PARAMS_VALID_ERROR,
                msg='active_url缺失'
            )
        active_url = input_json['active_url']

        if self.session.query(Account).filter_by(account=input_json['account']).first():
            return self.response_json(
                self.HttpErrorCode.PARAMS_VALID_ERROR,
                msg='账号已经被占用了'
            )

        if self.session.query(Account).filter_by(email=input_json['email']).first():
            return self.response_json(
                self.HttpErrorCode.PARAMS_VALID_ERROR,
                msg='邮箱已经被注册了'
            )

        input_json['password'] = bcrypt.hashpw(input_json['password'].encode('utf-8'), bcrypt.gensalt())

        account, errors = AccountSchema().load(input_json, partial=True)
        if errors:
            return self.response_json(self.HttpErrorCode.PARAMS_VALID_ERROR, msg=errors)
        account.save()

        active_token = ActiveAccountCache().create_active_token(account.id)
        redirect_url = '{active_host}/users/active_account&active_token={active_token}'.format(active_host=request.host, active_token=active_token)
        try:
            send_active_email(account.email, active_url, redirect_url)
        except Exception as e:
            # 如果邮件发送失败，则放弃邮件验证
            print(e)
            account.role = 'user'
            account.save()

        return self.response_json(self.HttpErrorCode.SUCCESS)



from flask import request, g

from seed.schema.base import BaseSchema
from seed.api.endpoints._base import RestfulBaseView
from seed.models import Bussiness as BussinessModel
from seed.models import BManager as BManagerModel
from seed.models import Account as AccountModel

from seed.utils.permissions import get_permission_datas_by_user
from seed.utils.helper import common_batch_crud
from seed.utils.auth import api_require_login


class BussinessSchema(BaseSchema):
    class Meta:
        model = BussinessModel


class BManagerSchema(BaseSchema):
    class Meta:
        model = BManagerModel
        include_fk = True


class Bussiness(RestfulBaseView):
    """ bussiness
    """
    model_class = BussinessModel
    schema_class = BussinessSchema
    decorators = [api_require_login]

    def get(self, bussiness_id=None):
        """ GET
        """
        if bussiness_id:
            data = self.session.query(
                self.model_class
            ).filter_by(id=bussiness_id).first()
            data = data.row2dict() if data else {}
            return self.response_json(self.HttpErrorCode.SUCCESS, data=data)
        else:
            # 所有业务
            permission_datas, un_permission_datas = get_permission_datas_by_user(g.user)

            bmanagers = self.session.query(
                BManagerModel.id, BManagerModel.bussiness_id,
                BManagerModel.user_id, AccountModel.name
            )\
                .join(AccountModel, BManagerModel.user_id == AccountModel.id)\
                .as_list()

            b_managers_map = {}
            for bmanager in bmanagers:
                b_managers_map.setdefault(bmanager['bussiness_id'], []).append(bmanager)

            for data in permission_datas:
                data['managers'] = b_managers_map.get(data['id'], [])
            for data in un_permission_datas:
                data['managers'] = b_managers_map.get(data['id'], [])

            datas = {'my_bussiness': permission_datas, 'other_bussiness': un_permission_datas}

            return self.response_json(self.HttpErrorCode.SUCCESS, data=datas)

    def post(self):
        """ POST
        """
        input_json = request.get_json()

        schema = self.schema_class()
        bussiness, errors = schema.load(input_json)

        if errors:
            return self.response_json(self.HttpErrorCode.PARAMS_VALID_ERROR, msg=errors)
        bussiness.save()

        # 管理员增改删
        managers = input_json.get('managers', [])
        for manager in managers:
            manager['bussiness_id'] = bussiness.id

        common_batch_crud(BManagerSchema, BManagerModel, managers)

        return self.response_json(self.HttpErrorCode.SUCCESS, data={'bussiness_id': bussiness.id})

    put = post



from flask import request

# from seed.api.common import _MethodView
from seed.utils.file import LocalFile
from seed.api.endpoints._base import RestfulBaseView, HttpMethods


class Files(RestfulBaseView):

    access_methods = [HttpMethods.POST]

    def post(self):
        upload_file = request.files['file']
        local_file = LocalFile()
        file_path = local_file.save(upload_file)
        if file_path:
            file_url = request.host_url + file_path.replace('\\', '/')
            return self.response_json(self.HttpErrorCode.SUCCESS, '上传成功', {'file_url': file_url})
        return self.response_json(self.HttpErrorCode.ERROR, '上传失败')



from seed.models._base import db

__all__ = ['AnalogdataDimensions', 'AnalogdataGamedatas']


class AnalogdataDimensions(db.Model):
    """ 测试数据-维度表
    """
    id = db.Column(db.Integer, primary_key=True)
    game_id = db.Column(db.Integer)
    game_name = db.Column(db.Text)
    platform_id = db.Column(db.Integer)
    platform_name = db.Column(db.Text)
    version_id = db.Column(db.Integer)
    version_name = db.Column(db.Text)


class AnalogdataGamedatas(db.Model):
    """ 测试数据-数据表
    """
    id = db.Column(db.Integer, primary_key=True)
    fdate = db.Column(db.Text)
    game_id = db.Column(db.Integer)
    platform_id = db.Column(db.Integer)
    version_id = db.Column(db.Integer)
    fregucnt = db.Column(db.Integer)
    factucnt = db.Column(db.Integer)
    fpayucnt = db.Column(db.Integer)
    fincome = db.Column(db.Float)



from . import db, BaseModel

__all__ = ['Bussiness']


class Bussiness(BaseModel):
    """ bussiness model class
    """

    name = db.Column(db.Text, nullable=False)
    description = db.Column(db.Text)

    bmanager = db.relationship('BManager', cascade="all,delete", backref="bussiness")
    databases = db.relationship('Databases', cascade="all,delete", backref="bussiness")
    filters = db.relationship('Filters', cascade="all,delete", backref="bussiness")
    menu = db.relationship('Menu', cascade="all,delete", backref="bussiness")
    role = db.relationship('Role', cascade="all,delete", backref="bussiness")
    role_menu = db.relationship('RoleMenu', cascade="all,delete", backref="bussiness")


from seed.drives import ALL_DRIVES


def get_db_instance(dtype, ip, port, name, user, password, **kwargs):
    # TODO 自动收集当前支持的驱动
    db = ALL_DRIVES[dtype](ip, int(port), name, user, password)
    return db



import json

from seed.libs.data_access.formatters.base import BaseFormatter


class LineFormatter(BaseFormatter):
    def __init__(self, *args, **kwargs):
        super(LineFormatter, self).__init__(*args, **kwargs)

    def format_data(self):
        # 得到分布项
        categories, series = self._get_chart_categories_and_series()

        # 获取series图形数据
        series = self._convert_series_data(categories, series)

        return {"categories": categories, "series": series}

    def _get_chart_categories_and_series(self):
        """
        获取categoies和series的维度
        """
        series = []

        category_columns, series_columns = self._get_chart_columns()

        # 从数据中获取到对应的categories然后通过数据的维度进行组合
        categories = self._get_categories_sort_by_dimensions(category_columns, series_columns, series)

        categories = self._get_categories_sort_by_indexs(categories, category_columns)

        if not series:
            series = [item['index'] for item in self.indexs]

        return categories, series

    def _get_categories_sort_by_dimensions(self, category_columns, series_columns, series):
        categories_dict = []
        for key in self.data.keys():
            key = json.loads(key)
            cateogry = {category_column: key[category_column] for category_column in category_columns}
            if cateogry not in categories_dict:
                categories_dict.append(cateogry)
            if series_columns:
                series_key = '-'.join([str(key[series_column]) for series_column in series_columns])
                if series_key not in series:
                    series.append(series_key)

        # categories排序
        for dimension in self.dimensions:
            if dimension['sort'] == 'desc':
                categories_dict.sort(key=lambda x: x[dimension['dimension']], reverse=True)
            if dimension['sort'] == 'asc':
                categories_dict.sort(key=lambda x: x[dimension['dimension']])

        categories = [
            '-'.join([str(category_dict[category_column]) for category_column in category_columns])
            for category_dict in categories_dict
        ]
        return categories

    def _get_chart_columns(self):
        category_columns, series_columns = [item['dimension'] for item in self.dimensions], []
        return category_columns, series_columns

    def _convert_series_data(self, categories, series):
        # 从数据中获取到趋势类型的数据
        category_columns, series_columns = self._get_chart_columns()
        series_map = {index['index']: index for index in self.indexs}

        middle_data = {}
        if series_columns:
            for key, value in self.data.items():
                key = json.loads(key)
                category_key = '-'.join([str(key[category_column]) for category_column in category_columns])
                if series_columns:
                    series_key = '-'.join([str(key[series_column]) for series_column in series_columns])
                    middle_data.setdefault(category_key, {})[series_key] = sum(value.values())
        else:
            middle_data = {
                '-'.join([str(json.loads(key)[category_column]) for category_column in category_columns]): value for
                key, value in self.data.items()
            }

        series_datas = []
        for serie in series:
            data = []
            for category in categories:
                data.append(middle_data.get(str(category), {}).get(serie, '-'))

            series_data = {'data': data, 'dim': serie, 'name': serie}
            series_data.update(series_map.get(serie, {}))
            series_datas.append(series_data)

        return series_datas

    def _get_categories_sort_by_indexs(self, categories, category_columns):
        if not len(categories):
            return categories

        if any(index['sort'] in ('desc', 'asc') for index in self.indexs):
            # 按指标进行排序
            categories = self._categories_sort_by_value(category_columns)

        return categories

    def _categories_sort_by_value(self, category_columns):
        middle_date = {}

        sort_index = None
        sort_reverse = None
        for index in self.indexs:
            if index['sort'] in ('desc', 'asc'):
                sort_index = index['index']
                sort_reverse = True if index['sort'] != 'desc' else False
                break

        for i, j in self.data.items():
            i = json.loads(i)
            middle_date.update(
                {'-'.join([str(i[category_column]) for category_column in category_columns]): j[sort_index]}
            )
        categories_data = sorted(middle_date.items(), key=lambda d: d[1] if d[1] else 0, reverse=sort_reverse)
        categories = [v[0] for v in categories_data]
        return categories



import bcrypt
from flask import request

from seed.models.account import Account
from seed.api.endpoints._base import RestfulBaseView, HttpMethods
from seed.cache.reset_password import ResetPasswordCache
from seed.utils.mail import send_reset_password_email


class ForgetPassword(RestfulBaseView):
    """ 忘记密码, 发送重置密码的邮件
    """
    access_methods = [HttpMethods.POST]
    url = 'forget_password'

    def post(self):
        input_json = request.get_json()
        if 'account' not in input_json:
            return self.response_json(
                self.HttpErrorCode.PARAMS_VALID_ERROR,
                msg='account参数缺失'
            )
        account = input_json['account']

        if 'reset_url' not in input_json:
            return self.response_json(
                self.HttpErrorCode.PARAMS_VALID_ERROR,
                msg='reset_url参数缺失'
            )
        reset_url = input_json['reset_url']

        account_session = self.session.query(Account)

        if '@' in account:
            account = account_session.filter_by(email=account).first()
        else:
            account = account_session.filter_by(account=account).first()

        if not account:
            return self.response_json(
                self.HttpErrorCode.PARAMS_VALID_ERROR,
                msg='该账户不存在，请查证后再试'
            )

        reset_token = ResetPasswordCache().create_active_token(account.id)
        reset_redirect_url = '{active_host}/users/reset_password&reset_token={reset_token}'.format(active_host=request.host, reset_token=reset_token)
        send_reset_password_email(account.email, reset_url, reset_redirect_url)

        return self.response_json(self.HttpErrorCode.SUCCESS)


class ResetPasssword(RestfulBaseView):
    """ 重置密码
    """
    access_methods = [HttpMethods.POST]
    url = 'reset_password'

    def post(self):
        input_json = request.get_json()
        if input_json['password'] != input_json['confirm_password']:
            return self.response_json(
                self.HttpErrorCode.PARAMS_VALID_ERROR,
                msg='密码和确认密码不一致'
            )

        if 'reset_token' not in input_json:
            return self.response_json(
                self.HttpErrorCode.PARAMS_VALID_ERROR,
                msg='reset_token参数不存在'
            )

        reset_account_id = ResetPasswordCache().get_user_by_active_token(input_json['reset_token'])
        account = self.session.query(Account).filter_by(id=reset_account_id).first()

        password = bcrypt.hashpw(input_json['password'].encode('utf-8'), bcrypt.gensalt())
        account.password = password
        account.save()

        return self.response_json(self.HttpErrorCode.SUCCESS)



import re
import json

from seed.libs.data_access.formatters.base import BaseFormatter


class TableFormatter(BaseFormatter):
    def __init__(self, *args, **kwargs):
        super(TableFormatter, self).__init__(*args, **kwargs)

    def format_data(self):
        # table_keys = self.title_flags + [*self.dims_set.keys()]

        self._ver_to_hor()

        table_keys = [item['dimension'] for item in self.dimensions] + [item['index'] for item in self.indexs]

        # 得到table的表格数据
        table_datas = self._convert_table_data(table_keys, self.data)

        # 对表格数据进行排序
        table_datas = self._sort_table_column(table_datas)

        # 处理比率类型数据
        # table_datas = self._convert_rate_data(table_keys, table_datas)

        # 得到table的显示项
        display_name = self._get_display_name(table_keys)

        return {"data": table_datas, "displayName": display_name}

    def _ver_to_hor(self):
        if not self.format_args.get('vth_columns'):
            return

        converted_data = {}

        index = self.indexs[0]['index']
        vth_columns = self.format_args.get('vth_columns')

        self.dimensions = [item for item in self.dimensions if item['dimension'] not in vth_columns]
        self.indexs = []

        for dimensions, values in self.data.items():
            dimensions = json.loads(dimensions)
            converted_dimensions, vth_dimensions = {}, []
            for key, value in dimensions.items():
                if key in vth_columns:
                    vth_dimensions.append(str(value))
                else:
                    converted_dimensions[key] = value
            if '-'.join(vth_dimensions) not in self.indexs:
                self.indexs.append('-'.join(vth_dimensions))
            converted_data.setdefault(json.dumps(converted_dimensions), {})['-'.join(vth_dimensions)] = values[index]

        self.indexs = [{'index': index, 'name': index} for index in self.indexs]
        self.data = converted_data

    def _convert_table_data(self, table_keys, datas):
        # 将数据从中间格式转换成表格格式
        table_datas = []

        for key, values in datas.items():
            values.update(json.loads(key))
            row = {}
            for key in table_keys:
                row[key] = values.get(key, '-')

            table_datas.append(row)

        return table_datas

    def _sort_table_column(self, data):
        # 进行table类型的排序

        order_fields = [item['dimension'] for item in self.dimensions] + [item['index'] for item in self.indexs]

        # 改成按多种排序
        data = sorted(
            data,
            key=lambda k: [(k[order_field] if k[order_field] else 0) for order_field in order_fields],
            reverse=True
        )

        return data

    # def _convert_rate_data(self, table_keys, datas):
    #     """处理比率类型指标的数据"""
    #     table_datas = []

    #     for data in datas:
    #         row = {}
    #         for index in self.indexs:
    #             row[key] = format_data(index['rate'], data.get(key, '-'))

    #         table_datas.append(row)

    #     return table_datas

    def _get_display_name(self, keys):
        # 获取displayName格式的数据显示
        titles = []

        dim_display_map = {}

        for item in self.dimensions:
            dim_display_map[item['dimension']] = item['name']

        for item in self.indexs:
            dim_display_map[item['index']] = item['name']

        for key in keys:
            titles.append({"name": key, "displayName": dim_display_map[key]})

        return titles



import json

from seed.cache.base import BaseCache


class RedisCache(BaseCache):
    max_size = 50 * 1024 * 1024

    def __init__(self, redis):
        self.redis_client = redis

    def set(self, key, value, timeout=None):
        key = self.make_key(key)
        v = json.dumps(value)
        if len(v) > self.max_size:
            raise ValueError("Cache value too large: %r %r" %(key, len(value)))
        if timeout:
            self.redis_client.set(key, v, int(timeout))
        else:
            self.redis_client.set(key, v)

    def get(self, key):
        key = self.make_key(key)
        result = self.redis_client.get(key)
        if result:
            result = json.loads(result)
        return result

    def delete(self, key):
        key = self.make_key(key)
        self.redis_client.delete(key)


import time
import bcrypt

from flask import request, make_response

from seed.models.account import Account
from seed.cache.session import SessionCache

from seed.api.endpoints._base import RestfulBaseView, HttpMethods


class Login(RestfulBaseView):
    """ 登录
    """
    access_methods = [HttpMethods.POST]

    def post(self):
        """ POST
        """
        input_json = request.get_json()
        if 'account' not in input_json:
            return self.response_json(self.HttpErrorCode.PARAMS_VALID_ERROR, '账号不能为空')

        if 'password' not in input_json:
            return self.response_json(self.HttpErrorCode.PARAMS_VALID_ERROR, '密码不能为空')

        account, password = input_json.get('account'), input_json.get('password')

        # 获取账号
        account = Account.query.filter_by(account=account).first()

        if not bcrypt.checkpw(password.encode('utf-8'), account.password.encode('utf-8')):
            return self.response_json(self.HttpErrorCode.AUTHORIZED_ERROR)

        # Cookie设置
        res = make_response(self.response_json(self.HttpErrorCode.SUCCESS))

        session_token = SessionCache().create_session(account.id)
        res.set_cookie(
            'session_token',
            session_token,
            expires=time.time()+24*60*60,
            domain=request.host
        )

        return res


class Logout(RestfulBaseView):
    """ 登出
    """
    access_methods = [HttpMethods.GET]

    def get(self):
        session_token = request.cookies.get('session_token', None)
        if not session_token:
            return self.response_json(self.HttpErrorCode.SUCCESS)

        SessionCache().delete(session_token)

        res = make_response(self.response_json(self.HttpErrorCode.SUCCESS))
        res.set_cookie('session_token', session_token, expires=0)

        return res



from seed.libs.data_access.formatters.line import LineFormatter


class LineStackFormatter(LineFormatter):
    def __init__(self, *args, **kwargs):
        super(LineFormatter, self).__init__(*args, **kwargs)

    def _get_chart_columns(self):
        category_columns = ['fdate']

        series_columns = [item['dimension'] for item in self.dimensions if item['dimension'] not in category_columns]

        return category_columns, series_columns



import random
import string

from flask import current_app

from seed.cache.redis import RedisCache


class ActiveAccountCache(RedisCache):
    def __init__(self):
        self.redis_client = current_app.cache
        self.middle_key = 'active_account'

    def make_key(self, key):
        key = ":".join([self.middle_key, str(key)])
        return super(ActiveAccountCache, self).make_key(key)

    def create_active_token(self, user_id):
        active_token = ''.join([random.choice(string.ascii_lowercase + string.digits) for key in range(48)])
        self.set(active_token, user_id, timeout=24 * 60 * 60)
        return active_token

    def get_user_by_active_token(self, active_token):
        return self.get(active_token)



import os
from setuptools import setup, find_packages

VERSION = '0.1.2'

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

def get_requirements(env):
    with open('requirements-{}.txt'.format(env)) as fp:
        return [x.strip() for x in fp.read().split('\n') if not x.startswith('#')]


install_requires = get_requirements('base')


def package_files(directory):
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        for filename in filenames:
            paths.append(os.path.join(path, filename))
    return paths


static_files = package_files(os.path.join('src', 'seed', 'static'))
static_files.extend(package_files(os.path.join('src', 'seed', 'data')))


setup(
    name="boyaa-seed",
    version=VERSION,
    auther="Boyaa DataCenter",
    auther_email="d@boyaa.com",
    description="seed data report system",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://seed.boyaa.com",
    install_requires=install_requires,
    packages=find_packages("src"),
    package_dir={
        "": "src"
    },
    data_files=static_files,
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "seed = seed.runner:main",
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Operating System :: OS Independent",
    ]
)



from flask import request, g

from seed.schema.base import BaseSchema
from seed.api.endpoints._base import RestfulBaseView
from seed.models.buserrole import BUserRole as BUserRoleModel
from seed.models.role import Role as RoleModel
from seed.utils.auth import api_require_login


class BUserRoleSchema(BaseSchema):
    class Meta:
        model = BUserRoleModel
        include_fk = True


class BUserRole(RestfulBaseView):
    """ 用户角色
    """
    model_class = BUserRoleModel
    schema_class = BUserRoleSchema
    decorators = [api_require_login]

    def get(self, model_id):
        query_session = self.session.query(self.model_class)
        datas = query_session.filter(self.model_class.user_id == model_id).all()
        datas = [row.row2dict() for row in datas] if datas else []

        roles = RoleModel.get_roles(g.bussiness_id)
        role_id_map = {role.id: role.role for role in roles}

        for data in datas:
            data['role'] = role_id_map.get(data['role_id'], '未知')

        return self.response_json(self.HttpErrorCode.SUCCESS, data=datas)

    def put(self, model_id):
        input_json = request.get_json()

        # 删除旧数据
        query_session = self.session.query(self.model_class)
        query_session.filter(self.model_class.user_id == model_id).delete()
        self.session.commit()

        # 添加新数据
        datas, errors = self.schema_class().load(input_json, many=True)
        if errors:
            return self.response_json(self.HttpErrorCode.PARAMS_VALID_ERROR, msg=errors)
        [data.save() for data in datas]

        datas = [row.row2dict() for row in datas] if datas else []

        roles = RoleModel.get_roles(g.bussiness_id)
        role_id_map = {role.id: role.role for role in roles}

        for data in datas:
            data['role'] = role_id_map.get(data['role_id'], '未知')

        return self.response_json(self.HttpErrorCode.SUCCESS, data=datas)


from ._base import db, BussinessModel

__all__ = ["BUserRole", ]


class BUserRole(BussinessModel):
    """ 用户权限模型
    """
    user_id = db.Column(db.Integer, nullable=False)
    role_id = db.Column(db.Integer, nullable=False)


import logging


def initialize_app(config):
    """
    initialize a application
    """

    configure_logging()


def configure_logging():
    """
    configure logging
    """
    from seed.conf import server
    logging.config.dictConfig(server.LOGGING)



from flask import request

# from seed.api.common import _MethodView
from seed.api.endpoints._base import RestfulBaseView, HttpMethods
from seed.drives import ALL_DRIVES


class DatabaseTypes(RestfulBaseView):
    url = 'database_types'

    access_methods = [HttpMethods.GET]

    def get(self):

        data = [{'value': drive, 'label': drive} for drive in ALL_DRIVES.keys()]
        return self.response_json(self.HttpErrorCode.SUCCESS, data=data)


from flask.views import MethodView
from flask import jsonify, request

from seed.utils.response import response, HttpErrorCode


class _MethodView(MethodView):
    """ MethodView base package
    """
    __abstract__ = True

    HttpErrorCode = HttpErrorCode

    def __init__(self, *args, **kwargs):
        super(_MethodView, self).__init__(*args, **kwargs)

    def response_json(self, code, msg=None, data={}):
        return jsonify(response(code, msg, data))

    @classmethod
    def register_api(cls, app):
        raise NotImplementedError()



import re

from flask import request

from seed.api.endpoints._base import RestfulBaseView, HttpMethods


class SqlFieldAnalysis(RestfulBaseView):
    url = 'sql_fields'

    access_methods = [HttpMethods.POST]

    def post(self):
        input_json = request.get_json()

        if 'sqls' not in input_json:
            return self.response_json(self.HttpErrorCode.PARAMS_VALID_ERROR, msg='获取SQL失败')

        sqls = input_json['sqls']

        sql_str = sqls.lower().replace('\n', ' ')
        select_fileds = sql_str[
            re.search("select\W+", sql_str, re.I).end():
            re.search("\W+from\W+", sql_str, re.I).start()
        ]
        reg = re.compile(r'\(.*?\) ', re.I)
        select_fileds = reg.sub(' ', select_fileds)
        # 分割字段
        select_fileds = select_fileds.split(',')
        fields = []
        for select_filed in select_fileds:
            # 再以空格分割
            filed_split = select_filed.split(' ')
            i = len(filed_split)
            while i:
                i = i-1
                # 获取最后一个不为空格的字段
                if filed_split[i]:
                    fields.append(str(filed_split[i]).strip())
                    break
        return self.response_json(self.HttpErrorCode.SUCCESS, data={'fields': fields})




class BaseDrive(object):
    def __init__(self, ip, port, name, user, password):
        self.ip = ip
        self.port = port
        self.name = name
        self.user = user
        self.password = password
        self._connect()

    def query(self, sql, params=[], retry_count=1):
        raise NotImplementedError("Need to implemented!")

    def execute(self, sql, params=[]):
        raise NotImplementedError("Need to implemented!")

    def test_connection(self):
        rows = self.query("SELECT 1")
        return len(rows) == 1

    def _connect(self):
        raise NotImplementedError("Need to implemented!")

    def _raise_retry_count(self, retry_count):
        if retry_count < 0:
            raise Exception("Retry time out")


class Row(dict):

    """访问对象那样访问dict,行结果"""

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError(name)
